import os

#from chromogenic.clean import remove_ldap, remove_vnc, remove_sensu
from chromogenic.common import prepare_chroot_env, remove_chroot_env,\
                                   run_command,\
                                   rebuild_ramdisk,\
                                   append_line_in_files,\
                                   prepend_line_in_files,\
                                   replace_line_in_files
from chromogenic.clean import mount_and_clean


def export_source(src_managerCls, src_manager_creds, export_kwargs):
    """
    Machine/BootableVolume --> begin_export
    Use the source manager to download a local image file
    Then start the export by passing image file
    """
    download_manager = src_managerCls(**src_manager_creds)
    download_kwargs = download_manager.download_image_args(**export_kwargs)
    download_location = download_manager.download_image(**download_kwargs)
    return begin_export(download_location, download_manager, export_kwargs)

def export_instance(src_managerCls, src_manager_creds, exportCls, export_kwargs):
    """
    Instance --> Snapshot --> begin_export
    Use the source manager to download a local image file
    Then start the export by passing the image to exportCls
    """
    #Download the image
    download_manager = src_managerCls(**src_manager_creds)
    download_kwargs = download_manager.download_instance_args(**export_kwargs)
    download_location = download_manager.download_instance(**download_kwargs)
    return begin_export(download_location, download_manager, exportCls, export_kwargs)


def begin_export(download_location, src_manager, export_kwargs):
    #Clean the image (Optional)
    download_dir = os.path.dirname(download_location)
    mount_point = os.path.join(download_dir, 'mount_point/')
    if not os.path.exists(mount_point):
        os.makedirs(mount_point)
    if export_kwargs.get('clean_image',True):
        mount_and_clean(
                download_location,
                mount_point,
                status_hook=getattr(src_manager, 'hook', None),
                method_hook=getattr(src_manager, 'clean_hook', None),
                **export_kwargs)
    export_kwargs['download_location'] = download_location

    return download_location

def add_virtualbox_support(mounted_path, image_path):
    """
    These configurations are specific to running virtualbox from an exported VM
    """
    #remove_ldap(mounted_path)
    #remove_vnc(mounted_path)
    #remove_sensu(mounted_path)

    add_gnome_support(mounted_path)

    #Touch to create a new module file
    new_mod_file = os.path.join(mounted_path, 'etc/modprobe.d/virtualbox')
    open(new_mod_file,'a').close()

    add_eth0_module(mounted_path)
    add_intel_soundcard(mounted_path)

    rebuild_ramdisk(mounted_path)


def add_gnome_support(mounted_path):
    """
    RHEL only at this point.
    TODO: Add ubuntu, then add deterine_distro code
    """
    prepare_chroot_env(mounted_path)
    run_command([
        "/usr/sbin/chroot", mounted_path, "/bin/bash", "-c", "yum groupinstall"
        " -y \"X Window System\" \"GNOME Desktop Environment\""])
    #Selinux was enabled in the process. lets fix that:
    selinux_conf = os.path.join(mounted_path, 'etc/sysconfig/selinux')
    sed_replace("SELINUX=enforcing", "SELINUX=disabled", selinux_conf)
    remove_chroot_env(mounted_path)

    #Make it the default on boot
    replace_line_file_list = [
         (":[0-6]:initdefault",":5:initdefault",
             "etc/inittab"),
    ]
    replace_line_in_files(replace_line_file_list, mounted_path)


def add_eth0_module(mounted_path):
    prepend_line_list = [
         ("alias eth0 e1000",
          "etc/modprobe.d/virtualbox"),
    ]
    prepend_line_in_files(prepend_line_list, mounted_path)


def add_intel_soundcard(mounted_path):
    append_line_list = [
        ("alias scsi_hostadapter1 ahci","etc/modprobe.d/modprobe.conf"),
        ("install pciehp /sbin/modprobe -q --ignore-install acpiphp; "
         "/bin/true","etc/modprobe.d/virtualbox"),
        ("alias snd-card-0 snd-intel8x0","etc/modprobe.d/modprobe.conf"),
        ("options snd-intel8x0 index=0","etc/modprobe.d/modprobe.conf"),
        ("options snd-card-0 index=0","etc/modprobe.d/modprobe.conf"),
        ("remove snd-intel8x0 { /usr/sbin/alsactl store 0 >/dev/null "
            "2>& 1 || : ; }; /sbin/modprobe -r --ignore-remove "
            "snd-intel8x0","etc/modprobe.d/virtualbox")
    ]
    append_line_in_files(append_line_list, mounted_path)

def remove_sensu(mounted_path):
    try:
        prepare_chroot_env(mounted_path)
        run_command(["/usr/sbin/chroot", mounted_path, 'yum',
                     'remove', '-qy', 'sensu'])
    finally:
        remove_chroot_env(mounted_path)

def remove_vnc(mounted_path):
    try:
        prepare_chroot_env(mounted_path)
        run_command(["/usr/sbin/chroot", mounted_path, 'yum',
            'remove', '-qy', 'realvnc-vnc-server'])
        #remove rpmsave.. to get rid of vnc for good.
        #["/usr/sbin/chroot", mounted_path, 'find', '/',
        #'-type', 'f', '-name', '*.rpmsave', '-exec', 'rm', '-f',
        #'{}', ';']
    finally:
        remove_chroot_env(mounted_path)



"""
Current logger version.
"""

from subprocess import Popen, PIPE
from os.path import abspath, dirname

VERSION = (0, 4, 8, 'dev', 0)

git_match = "(?P<git_flag>git://)\S*#egg="\
            "(?P<egg>[a-zA-Z0-9-]*[a-zA-Z])"\
            "(?P<opt_version_flag>-)?(?P<opt_version>[0-9][0-9.-]*[0-9])?(?P<dev_flag>-dev)?"
            # Version is optional
egg_match = "(?P<egg>\S.*[a-zA-Z])"\
            "(?P<opt_version_flag>[-=]+)?"\
            "(?P<opt_version>[0-9]*[0-9.-]*[0-9])?(?P<dev_flag>-dev)?"
            # Version is optional
def read_requirements(requirements_file):
    """
    Requirements files use two specific formats:
    packagename==1.3.1
    and
    git+git://github.com/abc/xyz.git#egg=packagename-1.3.1

    This function converts git to the bottom format
    and
    includes them as a dependency_link for setup.py
    """
    import re
    dependencies = []
    install_requires = []
    egg_regex = re.compile(egg_match)
    git_regex = re.compile(git_match)
    with open(requirements_file, 'r') as f:
        for line in f.read().split('\n'):
            # Skip empty spaces
            if not line:
                continue
            # Ignore comments.
            if line.lstrip().startswith("#"):
                continue
            # Read the line for version info
            r = git_regex.search(line)
            if not r:
                r = egg_regex.search(line)
            if not r:
                continue
            group = r.groupdict()
            if not group:
                continue
            #Dependencies will match git_flag
            if group.get('git_flag'):
                dependencies.append(line)
            #Requirements should be added for each line
            if group.get('opt_version') and group.get('egg'):
                install_requires.append("%s==%s%s" % (group['egg'], group['opt_version'],
                        '-dev' if group.get('dev_flag') else ''))
            elif group.get('egg'):
                install_requires.append("%s" % (group['egg']))
    return (dependencies, install_requires)


def write_requirements(requirements_file, new_file):
    (dependencies, install_requires) = read_requirements(requirements_file)
    with open(new_file,'w') as write_to:
        write_to.write("#Dependencies:\n")
        [write_to.write("%s\n" % line) for line in dependencies]
        write_to.write("#Requirements:\n")
        [write_to.write("%s\n" % line) for line in install_requires]
    return
        

def git_sha():
    loc = abspath(dirname(__file__))
    try:
        p = Popen(
            "cd \"%s\" && git log -1 --format=format:%%h" % loc,
            shell=True,
            stdout=PIPE,
            stderr=PIPE
        )
        return p.communicate()[0]
    except OSError:
        return None


def get_version(form='short'):
    """
    Returns the version string.

    Takes single argument ``form``, which should be one of the following
    strings:
    
    * ``short`` Returns major + minor branch version string with the format of
    B.b.t.
    * ``normal`` Returns human readable version string with the format of 
    B.b.t _type type_num.
    * ``verbose`` Returns a verbose version string with the format of
    B.b.t _type type_num@git_sha
    * ``all`` Returns a dict of all versions.
    """
    versions = {}
    branch = "%s.%s" % (VERSION[0], VERSION[1])
    tertiary = VERSION[2]
    type_ = VERSION[3]
    type_num = VERSION[4]
    
    versions["branch"] = branch
    v = versions["branch"]
    if tertiary:
        versions["tertiary"] = "." + str(tertiary)
        v += versions["tertiary"]
    versions['short'] = v
    if form is "short":
        return v
    v += " " + type_ + " " + str(type_num)
    versions["normal"] = v
    if form is "normal":
        return v
    v += " @" + git_sha()
    versions["verbose"] = v
    if form is "verbose":
        return v
    if form is "all":
        return versions




# -*- coding: utf-8 -*-
"""
Settings for the authentication app.
"""

from django.conf import settings
from django.test.signals import setting_changed
import sys
import os

APP_DIRECTORY = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, APP_DIRECTORY)
#NOTE: if 'SECRETS_MODULE' is enabled, chromogenic settings should be kept there.
if hasattr(settings, 'SECRETS_MODULE'):
    if hasattr(settings, 'CHROMOGENIC'):
        raise Exception(
            "Move definition of 'CHROMOGENIC' *OUT* of your local.py and "
            "into the file defined in SECRETS_MODULE")
    settings = getattr(settings, 'SECRETS_MODULE')


USER_SETTINGS = getattr(settings, 'CHROMOGENIC', {})


DEFAULTS =  {
    # General
    "SSH_KEY": "",
}

class ReadOnlyAttrDict(dict):
    __getattr__ = dict.__getitem__

new_settings = DEFAULTS.copy()
new_settings.update(USER_SETTINGS)
# This 'settings' instance will be used in the code
chromo_settings = ReadOnlyAttrDict(new_settings)


def reload_settings(*args, **kwargs):
    global chromo_settings
    setting_name, values = kwargs['setting'], kwargs['value']
    if setting_name == "CHROMOGENIC":
        defaults = DEFAULTS.copy()
        chromo_settings = ReadOnlyAttrDict(defaults.update(values))


setting_changed.connect(reload_settings)



import os
import logging
from chromogenic.common import mount_image, remove_files, fsck_image
from chromogenic.common import run_command, atmo_required_files
from chromogenic.common import copy_disk, create_empty_image
from chromogenic.clean import mount_and_clean
from chromogenic.common import prepare_chroot_env, remove_chroot_env

logger = logging.getLogger(__name__)

class BaseDriver():
    def parse_download_args(self, instance_id, **kwargs):
        raise NotImplementedError()
    def parse_upload_args(self, instance_id, **kwargs):
        raise NotImplementedError()
    def download_instance(self, instance_id, download_location, *args, **kwargs):
        raise NotImplementedError()
    def upload_local_image(self, image_location, image_name, *args, **kwargs):
        raise NotImplementedError()

    def create_image(self, instance_id, image_name, *args, **kwargs):
        """
        A 'Basic' create_image pattern. Download, Clean, Upload
        Return the new_image_id
        """
        download_args = self.parse_download_args(**kwargs)
        local_image_path = self.download_instance(instance_id, **download_args)
        mount_and_clean(
                local_image_path,
                status_hook=getattr(self, 'hook', None),
                method_hook=getattr(self, 'clean_hook',None),
                *args, **kwargs)
        upload_args = self.parse_upload_args(instance_id, **kwargs)
        new_image_id = self.upload_local_image(local_image_path, image_name, **upload_args)
        return new_image_id

    def _get_file_size_gb(self, filename):
        #TODO: Move to export.py
        import math
        byte_size = os.path.getsize(filename)
        one_gb = 1024**3
        gb_size = math.ceil( float(byte_size)/one_gb )
        return int(gb_size)

    def _copy_image(self, local_img_path, pad_size=1, ext='raw'):
        #Image is now ready to be placed on a bootable drive, then install grub-legacy
        image_size = self._get_file_size_gb(local_img_path)
        local_raw_path = local_img_path +  "." + ext
        create_empty_image(local_raw_path, ext,
                           image_size+pad_size,  # Add some empty space..
                           bootable=True)
        download_dir = os.path.dirname(local_img_path)
        mount_point = os.path.join(download_dir,"mount_point/")
        #copy the data
        copy_disk(old_image=local_img_path,
                  new_image=local_raw_path,
                  download_dir=download_dir)
        #TODO: Grow filesystem via OS??
        return local_raw_path


